package ru.dbotthepony.mc.otm.data

import com.google.common.collect.ImmutableList
import com.google.gson.*
import com.mojang.serialization.Codec
import com.mojang.serialization.DataResult
import com.mojang.serialization.Dynamic
import com.mojang.serialization.JsonOps
import com.mojang.serialization.codecs.RecordCodecBuilder
import it.unimi.dsi.fastutil.objects.ObjectArrayList
import net.minecraft.world.item.ItemStack
import net.minecraft.world.level.storage.loot.Deserializers
import net.minecraft.world.level.storage.loot.LootContext
import net.minecraft.world.level.storage.loot.LootPool
import net.minecraft.world.level.storage.loot.predicates.LootItemCondition
import net.minecraftforge.common.loot.IGlobalLootModifier
import net.minecraftforge.common.loot.LootModifier
import java.util.Arrays
import java.util.stream.Stream

class LootPoolAppender(conditions: Array<out LootItemCondition>, pools: Stream<LootPool>) : LootModifier(conditions) {
	private val pools = pools.collect(ImmutableList.toImmutableList())

	constructor(conditions: Array<out LootItemCondition>, pools: Collection<LootPool>) : this(conditions, pools.stream())
	constructor(conditions: Array<out LootItemCondition>, vararg pools: LootPool) : this(conditions, Arrays.stream(pools))

	override fun doApply(generatedLoot: ObjectArrayList<ItemStack>, context: LootContext): ObjectArrayList<ItemStack> {
		pools.forEach { it.addRandomItems(generatedLoot::add, context) }
		return generatedLoot
	}

	override fun codec(): Codec<out IGlobalLootModifier> {
		return CODEC
	}

	companion object {
		// same bro
		private fun getJson(it: Dynamic<Any?>): JsonElement {
			val value = it.value ?: throw NullPointerException("value was null")
			return value as? JsonElement ?: it.ops.convertTo(JsonOps.INSTANCE, it)
		}

		private val lootPoolCodec: Codec<List<LootPool>>

		init {
			val serializer = Deserializers.createLootTableSerializer().create()

			lootPoolCodec = Codec.list(Codec.PASSTHROUGH.flatXmap({
				try {
					DataResult.success(serializer.fromJson(getJson(it as Dynamic<Any?>), LootPool::class.java))
				} catch(err: JsonSyntaxException) {
					DataResult.error(err.message)
				}
			}, {
				try {
					DataResult.success(Dynamic(JsonOps.INSTANCE, serializer.toJsonTree(it)))
				} catch(err: JsonSyntaxException) {
					DataResult.error(err.message)
				}
			}))
		}

		val CODEC: Codec<LootPoolAppender> =
			RecordCodecBuilder.create {
				codecStart(it).and(
					lootPoolCodec.fieldOf("pools").forGetter(LootPoolAppender::pools)
				).apply(it, ::LootPoolAppender)
			}
	}
}
