package ru.dbotthepony.mc.otm.data

import com.google.common.collect.ImmutableList
import com.mojang.serialization.Codec
import com.mojang.serialization.codecs.RecordCodecBuilder
import it.unimi.dsi.fastutil.objects.ObjectArrayList
import net.minecraft.world.item.ItemStack
import net.minecraft.world.level.storage.loot.LootContext
import net.minecraft.world.level.storage.loot.predicates.LootItemCondition
import net.minecraftforge.common.loot.IGlobalLootModifier
import net.minecraftforge.common.loot.LootModifier
import java.util.*
import java.util.stream.Stream

class PlainLootAppender(conditions: Array<out LootItemCondition>, entries: Stream<Pair<ItemStack, Double>>) : LootModifier(conditions) {
	constructor(conditions: Array<out LootItemCondition>, vararg entries: Pair<ItemStack, Double>) : this(conditions, Arrays.stream(entries))
	constructor(conditions: Array<out LootItemCondition>, entries: Collection<Pair<ItemStack, Double>>) : this(conditions, entries.stream())

	private val entries = entries.map { it.first.copy() to it.second }.collect(ImmutableList.toImmutableList())

	override fun doApply(generatedLoot: ObjectArrayList<ItemStack>, context: LootContext): ObjectArrayList<ItemStack> {
		for ((item, chance) in entries) {
			if (context.random.nextDouble() <= chance) {
				generatedLoot.add(item.copy())
			}
		}

		return generatedLoot
	}

	override fun codec(): Codec<out IGlobalLootModifier> {
		return CODEC
	}

	companion object {
		private val paircodec: Codec<Pair<ItemStack, Double>> = RecordCodecBuilder.create {
			it.group(
				ItemStackCodec.fieldOf("item").forGetter { it.first },
				Codec.DOUBLE.fieldOf("chance").forGetter { it.second }
			).apply(it, ::Pair)
		}

		private val pairlistcodec = Codec.list(paircodec)

		val CODEC: Codec<PlainLootAppender> =
			RecordCodecBuilder.create {
				codecStart(it).and(
					pairlistcodec.fieldOf("entries").forGetter(PlainLootAppender::entries)
				).apply(it, ::PlainLootAppender)
			}
	}
}
